﻿using System.Collections.Generic;

namespace Parade.Metadata.Id3
{
	/// <summary>
	/// 定義済みのタグ ( ID3v2.2 ～ 2.4 ) に関する情報を提供します。
	/// </summary>
	public static class Id3DefinedTags
	{
		/// <summary>
		/// 定義済みのタグ ( ID3v2.2 ) に関する情報を提供します。
		/// </summary>
		public static class V22
		{
			/// <summary>
			/// クラスを初期化します。
			/// </summary>
			static V22()
			{
				V22.BUF = new Id3TagInfo( "BUF", Id3TagDataType.Binary                     );
				V22.CNT = new Id3TagInfo( "CNT", Id3TagDataType.Binary                     );
				V22.COM = new Id3TagInfo( "COM", Id3TagDataType.TextWithCommentAndLanguage );
				V22.CRA = new Id3TagInfo( "CRA", Id3TagDataType.BinaryWithComment          );
				V22.CRM = new Id3TagInfo( "CRM", Id3TagDataType.Binary                     );
				V22.ETC = new Id3TagInfo( "ETC", Id3TagDataType.Binary                     );
				V22.EQU = new Id3TagInfo( "EQU", Id3TagDataType.Binary                     );
				V22.GEO = new Id3TagInfo( "GEO", Id3TagDataType.Binary                     );
				V22.IPL = new Id3TagInfo( "IPL", Id3TagDataType.Text                       );
				V22.LNK = new Id3TagInfo( "LNK", Id3TagDataType.Link                       );
				V22.MCI = new Id3TagInfo( "MCI", Id3TagDataType.Binary                     );
				V22.MLL = new Id3TagInfo( "MLL", Id3TagDataType.Binary                     );
				V22.PIC = new Id3TagInfo( "PIC", Id3TagDataType.Binary                     );
				V22.POP = new Id3TagInfo( "POP", Id3TagDataType.BinaryWithComment          );
				V22.REV = new Id3TagInfo( "REV", Id3TagDataType.Binary                     );
				V22.RVA = new Id3TagInfo( "RVA", Id3TagDataType.Binary                     );
				V22.SLT = new Id3TagInfo( "SLT", Id3TagDataType.Binary                     );
				V22.STC = new Id3TagInfo( "STC", Id3TagDataType.Binary                     );
				V22.TAL = new Id3TagInfo( "TAL", Id3TagDataType.Text                       );
				V22.TBP = new Id3TagInfo( "TBP", Id3TagDataType.Text                       );
				V22.TCM = new Id3TagInfo( "TCM", Id3TagDataType.Text                       );
				V22.TCO = new Id3TagInfo( "TCO", Id3TagDataType.Text                       );
				V22.TCR = new Id3TagInfo( "TCR", Id3TagDataType.Text                       );
				V22.TDA = new Id3TagInfo( "TDA", Id3TagDataType.Text                       );
				V22.TDY = new Id3TagInfo( "TDY", Id3TagDataType.Text                       );
				V22.TEN = new Id3TagInfo( "TEN", Id3TagDataType.Text                       );
				V22.TFT = new Id3TagInfo( "TFT", Id3TagDataType.Text                       );
				V22.TIM = new Id3TagInfo( "TIM", Id3TagDataType.Text                       );
				V22.TKE = new Id3TagInfo( "TKE", Id3TagDataType.Text                       );
				V22.TLA = new Id3TagInfo( "TLA", Id3TagDataType.Text                       );
				V22.TLE = new Id3TagInfo( "TLE", Id3TagDataType.Text                       );
				V22.TMT = new Id3TagInfo( "TMT", Id3TagDataType.Text                       );
				V22.TOA = new Id3TagInfo( "TOA", Id3TagDataType.Text                       );
				V22.TOF = new Id3TagInfo( "TOF", Id3TagDataType.Text                       );
				V22.TOL = new Id3TagInfo( "TOL", Id3TagDataType.Text                       );
				V22.TOR = new Id3TagInfo( "TOR", Id3TagDataType.Text                       );
				V22.TOT = new Id3TagInfo( "TOT", Id3TagDataType.Text                       );
				V22.TP1 = new Id3TagInfo( "TP1", Id3TagDataType.Text                       );
				V22.TP2 = new Id3TagInfo( "TP2", Id3TagDataType.Text                       );
				V22.TP3 = new Id3TagInfo( "TP3", Id3TagDataType.Text                       );
				V22.TP4 = new Id3TagInfo( "TP4", Id3TagDataType.Text                       );
				V22.TPA = new Id3TagInfo( "TPA", Id3TagDataType.Text                       );
				V22.TPB = new Id3TagInfo( "TPB", Id3TagDataType.Text                       );
				V22.TRC = new Id3TagInfo( "TRC", Id3TagDataType.Text                       );
				V22.TRD = new Id3TagInfo( "TRD", Id3TagDataType.Text                       );
				V22.TRK = new Id3TagInfo( "TRK", Id3TagDataType.Text                       );
				V22.TSI = new Id3TagInfo( "TSI", Id3TagDataType.Text                       );
				V22.TSS = new Id3TagInfo( "TSS", Id3TagDataType.Text                       );
				V22.TT1 = new Id3TagInfo( "TT1", Id3TagDataType.Text                       );
				V22.TT2 = new Id3TagInfo( "TT2", Id3TagDataType.Text                       );
				V22.TT3 = new Id3TagInfo( "TT3", Id3TagDataType.Binary                     );
				V22.TXT = new Id3TagInfo( "TXT", Id3TagDataType.Text                       );
				V22.TXX = new Id3TagInfo( "TXX", Id3TagDataType.TextUserDefined            );
				V22.TYE = new Id3TagInfo( "TYE", Id3TagDataType.Text                       );
				V22.UFI = new Id3TagInfo( "UFI", Id3TagDataType.Binary                     );
				V22.ULT = new Id3TagInfo( "ULT", Id3TagDataType.TextWithCommentAndLanguage );
				V22.WAF = new Id3TagInfo( "WAF", Id3TagDataType.TextSimple                 );
				V22.WAR = new Id3TagInfo( "WAR", Id3TagDataType.TextSimple                 );
				V22.WAS = new Id3TagInfo( "WAS", Id3TagDataType.TextSimple                 );
				V22.WCM = new Id3TagInfo( "WCM", Id3TagDataType.TextSimple                 );
				V22.WCP = new Id3TagInfo( "WCP", Id3TagDataType.TextSimple                 );
				V22.WPB = new Id3TagInfo( "WPB", Id3TagDataType.TextSimple                 );
				V22.WXX = new Id3TagInfo( "WXX", Id3TagDataType.Binary                     );

				V22.Infos = new Dictionary< string, Id3TagInfo >()
				{
					{ V22.BUF.Id, V22.BUF },
					{ V22.CNT.Id, V22.CNT },
					{ V22.COM.Id, V22.COM },
					{ V22.CRA.Id, V22.CRA },
					{ V22.CRM.Id, V22.CRM },
					{ V22.ETC.Id, V22.ETC },
					{ V22.EQU.Id, V22.EQU },
					{ V22.GEO.Id, V22.GEO },
					{ V22.IPL.Id, V22.IPL },
					{ V22.LNK.Id, V22.LNK },
					{ V22.MCI.Id, V22.MCI },
					{ V22.MLL.Id, V22.MLL },
					{ V22.PIC.Id, V22.PIC },
					{ V22.POP.Id, V22.POP },
					{ V22.REV.Id, V22.REV },
					{ V22.RVA.Id, V22.RVA },
					{ V22.SLT.Id, V22.SLT },
					{ V22.STC.Id, V22.STC },
					{ V22.TAL.Id, V22.TAL },
					{ V22.TBP.Id, V22.TBP },
					{ V22.TCM.Id, V22.TCM },
					{ V22.TCO.Id, V22.TCO },
					{ V22.TCR.Id, V22.TCR },
					{ V22.TDA.Id, V22.TDA },
					{ V22.TDY.Id, V22.TDY },
					{ V22.TEN.Id, V22.TEN },
					{ V22.TFT.Id, V22.TFT },
					{ V22.TIM.Id, V22.TIM },
					{ V22.TKE.Id, V22.TKE },
					{ V22.TLA.Id, V22.TLA },
					{ V22.TLE.Id, V22.TLE },
					{ V22.TMT.Id, V22.TMT },
					{ V22.TOA.Id, V22.TOA },
					{ V22.TOF.Id, V22.TOF },
					{ V22.TOL.Id, V22.TOL },
					{ V22.TOR.Id, V22.TOR },
					{ V22.TOT.Id, V22.TOT },
					{ V22.TP1.Id, V22.TP1 },
					{ V22.TP2.Id, V22.TP2 },
					{ V22.TP3.Id, V22.TP3 },
					{ V22.TP4.Id, V22.TP4 },
					{ V22.TPA.Id, V22.TPA },
					{ V22.TPB.Id, V22.TPB },
					{ V22.TRC.Id, V22.TRC },
					{ V22.TRD.Id, V22.TRD },
					{ V22.TRK.Id, V22.TRK },
					{ V22.TSI.Id, V22.TSI },
					{ V22.TSS.Id, V22.TSS },
					{ V22.TT1.Id, V22.TT1 },
					{ V22.TT2.Id, V22.TT2 },
					{ V22.TT3.Id, V22.TT3 },
					{ V22.TXT.Id, V22.TXT },
					{ V22.TXX.Id, V22.TXX },
					{ V22.TYE.Id, V22.TYE },
					{ V22.UFI.Id, V22.UFI },
					{ V22.ULT.Id, V22.ULT },
					{ V22.WAF.Id, V22.WAF },
					{ V22.WAR.Id, V22.WAR },
					{ V22.WAS.Id, V22.WAS },
					{ V22.WCM.Id, V22.WCM },
					{ V22.WCP.Id, V22.WCP },
					{ V22.WPB.Id, V22.WPB },
					{ V22.WXX.Id, V22.WXX }
				};
			}

			/// <summary>
			/// 指定された名前を持つタグ情報を取得します。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
			internal static Id3TagInfo GetInfo( string id )
			{
				Id3TagInfo info;
				return ( V22.Infos.TryGetValue( id, out info ) ? info : null );
			}

			/// <summary>
			/// タグ情報のコレクションを取得します。
			/// </summary>
			/// <returns>タグ情報のコレクション。</returns>
			public static IEnumerable< Id3TagInfo > GetTags()
			{
				return V22.Infos.Values;
			}

			/// <summary>
			/// 指定された名前を持つタグが定義されていることを調べます。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>定義されている場合は true。それ以外は false。</returns>
			internal static bool HasInfo( string id )
			{
				return V22.Infos.ContainsKey( id );
			}

			/// <summary>
			/// 推奨バッファサイズを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo BUF { get; private set; }

			/// <summary>
			/// 再生回数を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo CNT { get; private set; }
			
			/// <summary>
			/// コメントを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo COM { get; private set; }

			/// <summary>
			/// オーディオの暗号化形式を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo CRA { get; private set; }

			/// <summary>
			/// 暗号化メタフレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo CRM { get; private set; }
			
			/// <summary>
			/// イベントタイムコードを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ETC { get; private set; }

			/// <summary>
			/// イコライザー情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo EQU { get; private set; }
		
			/// <summary>
			/// パッケージ化された一般的なオブジェクトを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo GEO { get; private set; }

			/// <summary>
			/// 協力者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo IPL { get; private set; }

			/// <summary>
			/// リンク情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo LNK { get; private set; }

			/// <summary>
			/// 音楽 CD の識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo MCI { get; private set; }

			/// <summary>
			/// MPEG ロケーションルックアップテーブルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo MLL { get; private set; }

			/// <summary>
			/// ジャケット画像を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo PIC { get; private set; }

			/// <summary>
			/// 人気 ( 1 ～ 255 の数値で表す評価 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo POP { get; private set; }

			/// <summary>
			/// リバーブ量を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo REV { get; private set; }

			/// <summary>
			/// 相対ボリュームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo RVA { get; private set; }

			/// <summary>
			/// タイムラインを持つ歌詞情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo SLT { get; private set; }

			/// <summary>
			/// タイムラインを持つテンポ ( Bit Per Minute ) 情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo STC { get; private set; }

			/// <summary>
			/// 曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TAL { get; private set; }

			/// <summary>
			/// テンポ ( Bit Per Minute ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TBP { get; private set; }

			/// <summary>
			/// 作曲者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCM { get; private set; }

			/// <summary>
			/// ジャンルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCO { get; private set; }

			/// <summary>
			/// 著作権情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCR { get; private set; }

			/// <summary>
			/// 録音された日付を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDA { get; private set; }

			/// <summary>
			/// プレイリストの遅延時間 ( 曲間の無音時間をスキップするための情報 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDY { get; private set; }

			/// <summary>
			/// ファイルのエンコードをおこなった個人または組織を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TEN { get; private set; }

			/// <summary>
			/// オーディオファイルの形式を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TFT { get; private set; }

			/// <summary>
			/// レコーディングに要した時間 ( 形式は HHMM、計 4 桁の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIM { get; private set; }

			/// <summary>
			/// 曲頭のキー ( C、Am、F♯ といった調の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TKE { get; private set; }

			/// <summary>
			/// 言語 ( ISO-639-2 として定義された 3 文字の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TLA { get; private set; }

			/// <summary>
			/// 演奏時間 ( ミリ秒単位 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TLE { get; private set; }

			/// <summary>
			/// メディア種別 ( 音楽 CD、レコード、...etc ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TMT { get; private set; }

			/// <summary>
			/// 原曲のアーティストを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOA { get; private set; }

			/// <summary>
			/// 元のファイル名を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOF { get; private set; }

			/// <summary>
			/// 原曲の作詞者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOL { get; private set; }

			/// <summary>
			/// 原曲のリリース年を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOR { get; private set; }

			/// <summary>
			/// 原曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOT { get; private set; }

			/// <summary>
			/// 主なアーティスト、演奏者などを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TP1 { get; private set; }

			/// <summary>
			/// レコーディングをおこなった演奏者の追加情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TP2 { get; private set; }

			/// <summary>
			/// 指揮者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TP3 { get; private set; }

			/// <summary>
			/// リミックスや編曲の担当者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TP4 { get; private set; }

			/// <summary>
			/// 曲の属するグループ内の番号 ( スラッシュ区切りの文字列 ) を示すタグ定義を取得します。
			/// 複数枚や複数の組曲で構成されるアルバムの場合、それらの総数と何番目に位置するかを表します。
			/// 例えば 3 枚組のアルバムにおいて、2 枚目に属する曲ならば "2/3" となります。
			/// </summary>
			public static Id3TagInfo TPA { get; private set; }

			/// <summary>
			/// 曲の発行元 ( レーベル ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPB { get; private set; }

			/// <summary>
			/// 国際標準レコーディングコード（ International Standard Recording Code ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRC { get; private set; }

			/// <summary>
			/// 録音日時を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRD { get; private set; }

			/// <summary>
			/// トラック番号 ( 書式は "番号/総数" ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRK { get; private set; }

			/// <summary>
			/// オーディオファイルのサイズ ( ID3v2 タグを除いたバイト数 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSI { get; private set; }

			/// <summary>
			/// エンコード設定を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSS { get; private set; }

			/// <summary>
			/// グループの説明を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TT1 { get; private set; }

			/// <summary>
			/// タイトルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TT2 { get; private set; }

			/// <summary>
			/// 副題やタイトルの補足説明を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TT3 { get; private set; }

			/// <summary>
			/// 作詞家を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TXT { get; private set; }

			/// <summary>
			/// ユーザー定義文字情報フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TXX { get; private set; }

			/// <summary>
			/// リリース年を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TYE { get; private set; }

			/// <summary>
			/// ファイルの一意な識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo UFI { get; private set; }

			/// <summary>
			/// 歌詞を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ULT { get; private set; }

			/// <summary>
			/// オーディオファイルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WAF { get; private set; }

			/// <summary>
			/// アーティストの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WAR { get; private set; }

			/// <summary>
			/// 音源の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WAS { get; private set; }

			/// <summary>
			/// 販売サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WCM { get; private set; }

			/// <summary>
			/// 著作情報サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WCP { get; private set; }

			/// <summary>
			/// レーベルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WPB { get; private set; }

			/// <summary>
			/// ユーザー定義 URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WXX { get; private set; }

			/// <summary>
			/// フレーム ID をキーとするタグ情報のディクショナリ。
			/// </summary>
			private static Dictionary< string, Id3TagInfo > Infos;
		}

		/// <summary>
		/// 定義済みのタグ ( ID3v2.3 ) に関する情報を提供します。
		/// </summary>
		public class V23
		{
			/// <summary>
			/// クラスを初期化します。
			/// </summary>
			static V23()
			{
				V23.ANEC = new Id3TagInfo( "ANEC", Id3TagDataType.BinaryWithComment          );
				V23.APIC = new Id3TagInfo( "APIC", Id3TagDataType.Binary                     );
				V23.COMM = new Id3TagInfo( "COMM", Id3TagDataType.TextWithCommentAndLanguage );
				V23.COMR = new Id3TagInfo( "COMR", Id3TagDataType.Binary                     );
				V23.ENCR = new Id3TagInfo( "ENCR", Id3TagDataType.BinaryWithComment          );
				V23.EQUA = new Id3TagInfo( "EQUA", Id3TagDataType.Binary                     );
				V23.ETCO = new Id3TagInfo( "ETCO", Id3TagDataType.Binary                     );
				V23.GEOB = new Id3TagInfo( "GEOB", Id3TagDataType.Binary                     );
				V23.GRID = new Id3TagInfo( "GRID", Id3TagDataType.BinaryWithComment          );
				V23.IPLS = new Id3TagInfo( "IPLS", Id3TagDataType.Text                       );
				V23.LINK = new Id3TagInfo( "LINK", Id3TagDataType.Link                       );
				V23.MCDI = new Id3TagInfo( "MCDI", Id3TagDataType.Binary                     );
				V23.MLLT = new Id3TagInfo( "MLLT", Id3TagDataType.Binary                     );
				V23.OWNE = new Id3TagInfo( "OWNE", Id3TagDataType.TextUserDefined            );
				V23.PRIV = new Id3TagInfo( "PRIV", Id3TagDataType.BinaryWithComment          );
				V23.PCNT = new Id3TagInfo( "PCNT", Id3TagDataType.Binary                     );
				V23.POPM = new Id3TagInfo( "POPM", Id3TagDataType.BinaryWithComment          );
				V23.POSS = new Id3TagInfo( "POSS", Id3TagDataType.Binary                     );
				V23.RBUF = new Id3TagInfo( "RBUF", Id3TagDataType.Binary                     );
				V23.RVAD = new Id3TagInfo( "RVAD", Id3TagDataType.Binary                     );
				V23.RVRB = new Id3TagInfo( "RVRB", Id3TagDataType.Binary                     );
				V23.SYLT = new Id3TagInfo( "SYLT", Id3TagDataType.Binary                     );
				V23.SYTC = new Id3TagInfo( "SYTC", Id3TagDataType.Binary                     );
				V23.TALB = new Id3TagInfo( "TALB", Id3TagDataType.Text                       );
				V23.TBPM = new Id3TagInfo( "TBPM", Id3TagDataType.Text                       );
				V23.TCOM = new Id3TagInfo( "TCOM", Id3TagDataType.Text                       );
				V23.TCON = new Id3TagInfo( "TCON", Id3TagDataType.Text                       );
				V23.TCOP = new Id3TagInfo( "TCOP", Id3TagDataType.Text                       );
				V23.TDAT = new Id3TagInfo( "TDAT", Id3TagDataType.Text                       );
				V23.TDLY = new Id3TagInfo( "TDLY", Id3TagDataType.Text                       );
				V23.TENC = new Id3TagInfo( "TENC", Id3TagDataType.Text                       );
				V23.TEXT = new Id3TagInfo( "TEXT", Id3TagDataType.Text                       );
				V23.TFLT = new Id3TagInfo( "TFLT", Id3TagDataType.Text                       );
				V23.TIME = new Id3TagInfo( "TIME", Id3TagDataType.Text                       );
				V23.TIT1 = new Id3TagInfo( "TIT1", Id3TagDataType.Text                       );
				V23.TIT2 = new Id3TagInfo( "TIT2", Id3TagDataType.Text                       );
				V23.TIT3 = new Id3TagInfo( "TIT3", Id3TagDataType.Text                       );
				V23.TKEY = new Id3TagInfo( "TKEY", Id3TagDataType.Text                       );
				V23.TLAN = new Id3TagInfo( "TLAN", Id3TagDataType.Text                       );
				V23.TLEN = new Id3TagInfo( "TLEN", Id3TagDataType.Text                       );
				V23.TMED = new Id3TagInfo( "TMED", Id3TagDataType.Text                       );
				V23.TOAL = new Id3TagInfo( "TOAL", Id3TagDataType.Text                       );
				V23.TOFN = new Id3TagInfo( "TOFN", Id3TagDataType.Text                       );
				V23.TOLY = new Id3TagInfo( "TOLY", Id3TagDataType.Text                       );
				V23.TOPE = new Id3TagInfo( "TOPE", Id3TagDataType.Text                       );
				V23.TORY = new Id3TagInfo( "TORY", Id3TagDataType.Text                       );
				V23.TOWN = new Id3TagInfo( "TOWN", Id3TagDataType.Text                       );
				V23.TPE1 = new Id3TagInfo( "TPE1", Id3TagDataType.Text                       );
				V23.TPE2 = new Id3TagInfo( "TPE2", Id3TagDataType.Text                       );
				V23.TPE3 = new Id3TagInfo( "TPE3", Id3TagDataType.Text                       );
				V23.TPE4 = new Id3TagInfo( "TPE4", Id3TagDataType.Text                       );
				V23.TPOS = new Id3TagInfo( "TPOS", Id3TagDataType.Text                       );
				V23.TPUB = new Id3TagInfo( "TPUB", Id3TagDataType.Text                       );
				V23.TRCK = new Id3TagInfo( "TRCK", Id3TagDataType.Text                       );
				V23.TRDA = new Id3TagInfo( "TRDA", Id3TagDataType.Text                       );
				V23.TRSN = new Id3TagInfo( "TRSN", Id3TagDataType.Text                       );
				V23.TRSO = new Id3TagInfo( "TRSO", Id3TagDataType.Text                       );
				V23.TSIZ = new Id3TagInfo( "TSIZ", Id3TagDataType.Text                       );
				V23.TSRC = new Id3TagInfo( "TSRC", Id3TagDataType.Text                       );
				V23.TSSE = new Id3TagInfo( "TSSE", Id3TagDataType.Text                       );
				V23.TYER = new Id3TagInfo( "TYER", Id3TagDataType.Text                       );
				V23.TXXX = new Id3TagInfo( "TXXX", Id3TagDataType.TextUserDefined            );
				V23.UFID = new Id3TagInfo( "UFID", Id3TagDataType.BinaryWithComment          );
				V23.USER = new Id3TagInfo( "USER", Id3TagDataType.TextWithLanguage           );
				V23.USLT = new Id3TagInfo( "USLT", Id3TagDataType.TextWithCommentAndLanguage );
				V23.WCOM = new Id3TagInfo( "WCOM", Id3TagDataType.TextSimple                 );
				V23.WCOP = new Id3TagInfo( "WCOP", Id3TagDataType.TextSimple                 );
				V23.WOAF = new Id3TagInfo( "WOAF", Id3TagDataType.TextSimple                 );
				V23.WOAR = new Id3TagInfo( "WOAR", Id3TagDataType.TextSimple                 );
				V23.WOAS = new Id3TagInfo( "WOAS", Id3TagDataType.TextSimple                 );
				V23.WORS = new Id3TagInfo( "WORS", Id3TagDataType.TextSimple                 );
				V23.WPAY = new Id3TagInfo( "WPAY", Id3TagDataType.TextSimple                 );
				V23.WPUB = new Id3TagInfo( "WPUB", Id3TagDataType.TextSimple                 );
				V23.WXXX = new Id3TagInfo( "WXXX", Id3TagDataType.TextUserDefined            );

				V23.Infos = new Dictionary< string, Id3TagInfo >()
				{
					{ V23.ANEC.Id, V23.ANEC },
					{ V23.APIC.Id, V23.APIC },
					{ V23.COMM.Id, V23.COMM },
					{ V23.COMR.Id, V23.COMR },
					{ V23.ENCR.Id, V23.ENCR },
					{ V23.EQUA.Id, V23.EQUA },
					{ V23.ETCO.Id, V23.ETCO },
					{ V23.GEOB.Id, V23.GEOB },
					{ V23.GRID.Id, V23.GRID },
					{ V23.IPLS.Id, V23.IPLS },
					{ V23.LINK.Id, V23.LINK },
					{ V23.MCDI.Id, V23.MCDI },
					{ V23.MLLT.Id, V23.MLLT },
					{ V23.OWNE.Id, V23.OWNE },
					{ V23.PRIV.Id, V23.PRIV },
					{ V23.PCNT.Id, V23.PCNT },
					{ V23.POPM.Id, V23.POPM },
					{ V23.POSS.Id, V23.POSS },
					{ V23.RBUF.Id, V23.RBUF },
					{ V23.RVAD.Id, V23.RVAD },
					{ V23.RVRB.Id, V23.RVRB },
					{ V23.SYLT.Id, V23.SYLT },
					{ V23.SYTC.Id, V23.SYTC },
					{ V23.TALB.Id, V23.TALB },
					{ V23.TBPM.Id, V23.TBPM },
					{ V23.TCOM.Id, V23.TCOM },
					{ V23.TCON.Id, V23.TCON },
					{ V23.TCOP.Id, V23.TCOP },
					{ V23.TDAT.Id, V23.TDAT },
					{ V23.TDLY.Id, V23.TDLY },
					{ V23.TENC.Id, V23.TENC },
					{ V23.TEXT.Id, V23.TEXT },
					{ V23.TFLT.Id, V23.TFLT },
					{ V23.TIME.Id, V23.TIME },
					{ V23.TIT1.Id, V23.TIT1 },
					{ V23.TIT2.Id, V23.TIT2 },
					{ V23.TIT3.Id, V23.TIT3 },
					{ V23.TKEY.Id, V23.TKEY },
					{ V23.TLAN.Id, V23.TLAN },
					{ V23.TLEN.Id, V23.TLEN },
					{ V23.TMED.Id, V23.TMED },
					{ V23.TOAL.Id, V23.TOAL },
					{ V23.TOFN.Id, V23.TOFN },
					{ V23.TOLY.Id, V23.TOLY },
					{ V23.TOPE.Id, V23.TOPE },
					{ V23.TORY.Id, V23.TORY },
					{ V23.TOWN.Id, V23.TOWN },
					{ V23.TPE1.Id, V23.TPE1 },
					{ V23.TPE2.Id, V23.TPE2 },
					{ V23.TPE3.Id, V23.TPE3 },
					{ V23.TPE4.Id, V23.TPE4 },
					{ V23.TPOS.Id, V23.TPOS },
					{ V23.TPUB.Id, V23.TPUB },
					{ V23.TRCK.Id, V23.TRCK },
					{ V23.TRDA.Id, V23.TRDA },
					{ V23.TRSN.Id, V23.TRSN },
					{ V23.TRSO.Id, V23.TRSO },
					{ V23.TSIZ.Id, V23.TSIZ },
					{ V23.TSRC.Id, V23.TSRC },
					{ V23.TSSE.Id, V23.TSSE },
					{ V23.TYER.Id, V23.TYER },
					{ V23.TXXX.Id, V23.TXXX },
					{ V23.UFID.Id, V23.UFID },
					{ V23.USER.Id, V23.USER },
					{ V23.USLT.Id, V23.USLT },
					{ V23.WCOM.Id, V23.WCOM },
					{ V23.WCOP.Id, V23.WCOP },
					{ V23.WOAF.Id, V23.WOAF },
					{ V23.WOAR.Id, V23.WOAR },
					{ V23.WOAS.Id, V23.WOAS },
					{ V23.WORS.Id, V23.WORS },
					{ V23.WPAY.Id, V23.WPAY },
					{ V23.WPUB.Id, V23.WPUB },
					{ V23.WXXX.Id, V23.WXXX }
				};
			}
		
			/// <summary>
			/// 指定された名前を持つタグ情報を取得します。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
			internal static Id3TagInfo GetInfo( string id )
			{
				Id3TagInfo info;
				return ( V23.Infos.TryGetValue( id, out info ) ? info : null );
			}

			/// <summary>
			/// タグ情報のコレクションを取得します。
			/// </summary>
			/// <returns>タグ情報のコレクション。</returns>
			public static IEnumerable< Id3TagInfo > GetTags()
			{
				return V23.Infos.Values;
			}

			/// <summary>
			/// 指定された名前を持つタグが定義されていることを調べます。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>定義されている場合は true。それ以外は false。</returns>
			internal static bool HasInfo( string id )
			{
				return V23.Infos.ContainsKey( id );
			}
			
			/// <summary>
			/// オーディオの暗号化形式を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ANEC { get; private set; }

			/// <summary>
			/// ジャケット画像を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo APIC { get; private set; }

			/// <summary>
			/// コメントを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo COMM { get; private set; }

			/// <summary>
			/// コマーシャル フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo COMR { get; private set; }

			/// <summary>
			/// 暗号化手法を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ENCR { get; private set; }

			/// <summary>
			/// イコライザー情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo EQUA { get; private set; }

			/// <summary>
			/// イベント タイム コードを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ETCO { get; private set; }

			/// <summary>
			/// パッケージ化された一般的なオブジェクトを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo GEOB { get; private set; }

			/// <summary>
			/// グループ識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo GRID { get; private set; }

			/// <summary>
			/// 協力者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo IPLS { get; private set; }

			/// <summary>
			/// リンク情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo LINK { get; private set; }

			/// <summary>
			/// 音楽 CD の識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo MCDI { get; private set; }

			/// <summary>
			/// MPEG ロケーション ルックアップ テーブルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo MLLT { get; private set; }

			/// <summary>
			/// 所有権フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo OWNE { get; private set; }

			/// <summary>
			/// プライベート フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo PRIV { get; private set; }

			/// <summary>
			/// 再生回数を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo PCNT { get; private set; }

			/// <summary>
			/// 人気 ( 1 ～ 255 の数値で表す評価 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo POPM { get; private set; }

			/// <summary>
			/// 同期位置フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo POSS { get; private set; }

			/// <summary>
			/// 推奨バッファサイズを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo RBUF { get; private set; }

			/// <summary>
			/// 相対ボリュームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo RVAD { get; private set; }

			/// <summary>
			/// リバーブ量を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo RVRB { get; private set; }

			/// <summary>
			/// タイムラインを持つ歌詞情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo SYLT { get; private set; }

			/// <summary>
			/// タイムラインを持つテンポ ( Bit Per Minute ) 情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo SYTC { get; private set; }

			/// <summary>
			/// 曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TALB { get; private set; }

			/// <summary>
			/// テンポ ( Bit Per Minute ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TBPM { get; private set; }

			/// <summary>
			/// 作曲者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCOM { get; private set; }

			/// <summary>
			/// ジャンルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCON { get; private set; }

			/// <summary>
			/// 著作権情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCOP { get; private set; }

			/// <summary>
			/// 録音された日付を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDAT { get; private set; }

			/// <summary>
			/// プレイリストの遅延時間 ( 曲間の無音時間をスキップするための情報 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDLY { get; private set; }

			/// <summary>
			/// ファイルのエンコードをおこなった個人または組織を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TENC { get; private set; }

			/// <summary>
			/// 作詞家を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TEXT { get; private set; }

			/// <summary>
			/// オーディオファイルの形式を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TFLT { get; private set; }

			/// <summary>
			/// レコーディングに要した時間 ( 形式は HHMM、計 4 桁の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIME { get; private set; }

			/// <summary>
			/// グループの説明を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIT1 { get; private set; }

			/// <summary>
			/// タイトルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIT2 { get; private set; }

			/// <summary>
			/// 副題やタイトルの補足説明を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIT3 { get; private set; }

			/// <summary>
			/// 曲頭のキー ( C、Am、F♯ といった調の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TKEY { get; private set; }

			/// <summary>
			/// 言語 ( ISO-639-2 として定義された 3 文字の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TLAN { get; private set; }

			/// <summary>
			/// 演奏時間 ( ミリ秒単位 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TLEN { get; private set; }

			/// <summary>
			/// メディア種別 ( 音楽 CD、レコード、...etc ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TMED { get; private set; }

			/// <summary>
			/// 原曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOAL { get; private set; }

			/// <summary>
			/// 元のファイル名を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOFN { get; private set; }

			/// <summary>
			/// 原曲の作詞者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOLY { get; private set; }

			/// <summary>
			/// 原曲のアーティストを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOPE { get; private set; }

			/// <summary>
			/// 原曲のリリース年を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TORY { get; private set; }

			/// <summary>
			/// ファイルの所有者・ライセンシーを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOWN { get; private set; }

			/// <summary>
			/// 主なアーティスト、演奏者などを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE1 { get; private set; }

			/// <summary>
			/// レコーディングをおこなった演奏者の追加情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE2 { get; private set; }

			/// <summary>
			/// 指揮者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE3 { get; private set; }

			/// <summary>
			/// リミックスや編曲の担当者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE4 { get; private set; }

			/// <summary>
			/// 曲の属するグループ内の番号 ( スラッシュ区切りの文字列 ) を示すタグ定義を取得します。
			/// 複数枚や複数の組曲で構成されるアルバムの場合、それらの総数と何番目に位置するかを表します。
			/// 例えば 3 枚組のアルバムにおいて、2 枚目に属する曲ならば "2/3" となります。
			/// </summary>
			public static Id3TagInfo TPOS { get; private set; }

			/// <summary>
			/// 曲の発行元 ( レーベル ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPUB { get; private set; }

			/// <summary>
			/// トラック番号 ( 書式は "番号/総数" ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRCK { get; private set; }

			/// <summary>
			/// 録音日時を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRDA { get; private set; }

			/// <summary>
			/// インターネットラジオ局を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRSN { get; private set; }

			/// <summary>
			/// インターネットラジオ局の所有者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRSO { get; private set; }

			/// <summary>
			/// オーディオファイルのサイズ ( ID3v2 タグを除いたバイト数 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSIZ { get; private set; }

			/// <summary>
			/// 国際標準レコーディングコード（ International Standard Recording Code ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSRC { get; private set; }

			/// <summary>
			/// エンコード設定を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSSE { get; private set; }

			/// <summary>
			/// リリース年を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TYER { get; private set; }

			/// <summary>
			/// ユーザー定義文字情報フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TXXX { get; private set; }

			/// <summary>
			/// ファイルの一意な識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo UFID { get; private set; }

			/// <summary>
			/// 使用条件を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo USER { get; private set; }

			/// <summary>
			/// 歌詞を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo USLT { get; private set; }

			/// <summary>
			/// 販売サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WCOM { get; private set; }

			/// <summary>
			/// 著作情報サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WCOP { get; private set; }

			/// <summary>
			/// オーディオファイルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WOAF { get; private set; }

			/// <summary>
			/// アーティストの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WOAR { get; private set; }

			/// <summary>
			/// 音源の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WOAS { get; private set; }

			/// <summary>
			/// インターネットラジオ局の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WORS { get; private set; }

			/// <summary>
			/// 支払い方法を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WPAY { get; private set; }

			/// <summary>
			/// レーベルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WPUB { get; private set; }

			/// <summary>
			/// ユーザー定義 URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WXXX { get; private set; }

			/// <summary>
			/// フレーム ID をキーとするタグ情報のディクショナリ。
			/// </summary>
			private static Dictionary< string, Id3TagInfo > Infos;
		}

		/// <summary>
		/// 定義済みのタグ ( ID3v2.4 ) に関する情報を提供します。
		/// </summary>
		public class V24
		{
			/// <summary>
			/// クラスを初期化します。
			/// </summary>
			static V24()
			{
				V24.ANEC = new Id3TagInfo( "ANEC", Id3TagDataType.BinaryWithComment          );
				V24.APIC = new Id3TagInfo( "APIC", Id3TagDataType.Binary                     );
				V24.ASPI = new Id3TagInfo( "ASPI", Id3TagDataType.Binary                     );
				V24.COMM = new Id3TagInfo( "COMM", Id3TagDataType.TextWithCommentAndLanguage );
				V24.COMR = new Id3TagInfo( "COMR", Id3TagDataType.Binary                     );
				V24.ENCR = new Id3TagInfo( "ENCR", Id3TagDataType.BinaryWithComment          );
				V24.EQU2 = new Id3TagInfo( "EQU2", Id3TagDataType.Binary                     );
				V24.ETCO = new Id3TagInfo( "ETCO", Id3TagDataType.Binary                     );
				V24.GEOB = new Id3TagInfo( "GEOB", Id3TagDataType.Binary                     );
				V24.GRID = new Id3TagInfo( "GRID", Id3TagDataType.BinaryWithComment          );
				V24.LINK = new Id3TagInfo( "LINK", Id3TagDataType.Link                       );
				V24.MCDI = new Id3TagInfo( "MCDI", Id3TagDataType.Binary                     );
				V24.MLLT = new Id3TagInfo( "MLLT", Id3TagDataType.Binary                     );
				V24.OWNE = new Id3TagInfo( "OWNE", Id3TagDataType.TextUserDefined            );
				V24.PRIV = new Id3TagInfo( "PRIV", Id3TagDataType.BinaryWithComment          );
				V24.PCNT = new Id3TagInfo( "PCNT", Id3TagDataType.Binary                     );
				V24.POPM = new Id3TagInfo( "POPM", Id3TagDataType.BinaryWithComment          );
				V24.POSS = new Id3TagInfo( "POSS", Id3TagDataType.Binary                     );
				V24.RBUF = new Id3TagInfo( "RBUF", Id3TagDataType.Binary                     );
				V24.RVA2 = new Id3TagInfo( "RVA2", Id3TagDataType.Binary                     );
				V24.RVRB = new Id3TagInfo( "RVRB", Id3TagDataType.Binary                     );
				V24.SEEK = new Id3TagInfo( "SEEK", Id3TagDataType.BinaryWithSize             );
				V24.SIGN = new Id3TagInfo( "SIGN", Id3TagDataType.Binary                     );
				V24.SYLT = new Id3TagInfo( "SYLT", Id3TagDataType.Binary                     );
				V24.SYTC = new Id3TagInfo( "SYTC", Id3TagDataType.Binary                     );
				V24.TALB = new Id3TagInfo( "TALB", Id3TagDataType.Text                       );
				V24.TBPM = new Id3TagInfo( "TBPM", Id3TagDataType.Text                       );
				V24.TCOM = new Id3TagInfo( "TCOM", Id3TagDataType.Text                       );
				V24.TCON = new Id3TagInfo( "TCON", Id3TagDataType.Text                       );
				V24.TCOP = new Id3TagInfo( "TCOP", Id3TagDataType.Text                       );
				V24.TDEN = new Id3TagInfo( "TDEN", Id3TagDataType.Text                       );
				V24.TDLY = new Id3TagInfo( "TDLY", Id3TagDataType.Text                       );
				V24.TDOR = new Id3TagInfo( "TDOR", Id3TagDataType.Text                       );
				V24.TDRC = new Id3TagInfo( "TDRC", Id3TagDataType.Text                       );
				V24.TDRL = new Id3TagInfo( "TDRL", Id3TagDataType.Text                       );
				V24.TDTG = new Id3TagInfo( "TDTG", Id3TagDataType.Text                       );
				V24.TENC = new Id3TagInfo( "TENC", Id3TagDataType.Text                       );
				V24.TEXT = new Id3TagInfo( "TEXT", Id3TagDataType.Text                       );
				V24.TFLT = new Id3TagInfo( "TFLT", Id3TagDataType.Text                       );
				V24.TIPL = new Id3TagInfo( "TIPL", Id3TagDataType.Text                       );
				V24.TIME = new Id3TagInfo( "TIME", Id3TagDataType.Text                       );
				V24.TIT1 = new Id3TagInfo( "TIT1", Id3TagDataType.Text                       );
				V24.TIT2 = new Id3TagInfo( "TIT2", Id3TagDataType.Text                       );
				V24.TIT3 = new Id3TagInfo( "TIT3", Id3TagDataType.Text                       );
				V24.TKEY = new Id3TagInfo( "TKEY", Id3TagDataType.Text                       );
				V24.TLAN = new Id3TagInfo( "TLAN", Id3TagDataType.Text                       );
				V24.TLEN = new Id3TagInfo( "TLEN", Id3TagDataType.Text                       );
				V24.TMCL = new Id3TagInfo( "TMCL", Id3TagDataType.Text                       );
				V24.TMED = new Id3TagInfo( "TMED", Id3TagDataType.Text                       );
				V24.TMOO = new Id3TagInfo( "TMOO", Id3TagDataType.Text                       );
				V24.TOAL = new Id3TagInfo( "TOAL", Id3TagDataType.Text                       );
				V24.TOFN = new Id3TagInfo( "TOFN", Id3TagDataType.Text                       );
				V24.TOLY = new Id3TagInfo( "TOLY", Id3TagDataType.Text                       );
				V24.TOPE = new Id3TagInfo( "TOPE", Id3TagDataType.Text                       );
				V24.TOWN = new Id3TagInfo( "TOWN", Id3TagDataType.Text                       );
				V24.TPE1 = new Id3TagInfo( "TPE1", Id3TagDataType.Text                       );
				V24.TPE2 = new Id3TagInfo( "TPE2", Id3TagDataType.Text                       );
				V24.TPE3 = new Id3TagInfo( "TPE3", Id3TagDataType.Text                       );
				V24.TPE4 = new Id3TagInfo( "TPE4", Id3TagDataType.Text                       );
				V24.TPOS = new Id3TagInfo( "TPOS", Id3TagDataType.Text                       );
				V24.TPRO = new Id3TagInfo( "TPRO", Id3TagDataType.Text                       );
				V24.TPUB = new Id3TagInfo( "TPUB", Id3TagDataType.Text                       );
				V24.TRCK = new Id3TagInfo( "TRCK", Id3TagDataType.Text                       );
				V24.TRSN = new Id3TagInfo( "TRSN", Id3TagDataType.Text                       );
				V24.TRSO = new Id3TagInfo( "TRSO", Id3TagDataType.Text                       );
				V24.TSOA = new Id3TagInfo( "TSOA", Id3TagDataType.Text                       );
				V24.TSOP = new Id3TagInfo( "TSOP", Id3TagDataType.Text                       );
				V24.TSOT = new Id3TagInfo( "TSOT", Id3TagDataType.Text                       );
				V24.TSRC = new Id3TagInfo( "TSRC", Id3TagDataType.Text                       );
				V24.TSSE = new Id3TagInfo( "TSSE", Id3TagDataType.Text                       );
				V24.TSST = new Id3TagInfo( "TSST", Id3TagDataType.Text                       );
				V24.TXXX = new Id3TagInfo( "TXXX", Id3TagDataType.TextUserDefined            );
				V24.UFID = new Id3TagInfo( "UFID", Id3TagDataType.BinaryWithComment          );
				V24.USER = new Id3TagInfo( "USER", Id3TagDataType.TextWithLanguage           );
				V24.USLT = new Id3TagInfo( "USLT", Id3TagDataType.TextWithCommentAndLanguage );
				V24.WCOM = new Id3TagInfo( "WCOM", Id3TagDataType.TextSimple                 );
				V24.WCOP = new Id3TagInfo( "WCOP", Id3TagDataType.TextSimple                 );
				V24.WOAF = new Id3TagInfo( "WOAF", Id3TagDataType.TextSimple                 );
				V24.WOAR = new Id3TagInfo( "WOAR", Id3TagDataType.TextSimple                 );
				V24.WOAS = new Id3TagInfo( "WOAS", Id3TagDataType.TextSimple                 );
				V24.WORS = new Id3TagInfo( "WORS", Id3TagDataType.TextSimple                 );
				V24.WPAY = new Id3TagInfo( "WPAY", Id3TagDataType.TextSimple                 );
				V24.WPUB = new Id3TagInfo( "WPUB", Id3TagDataType.TextSimple                 );
				V24.WXXX = new Id3TagInfo( "WXXX", Id3TagDataType.TextUserDefined            );

				V24.Infos = new Dictionary< string, Id3TagInfo >()
				{
					{ V24.ANEC.Id, V24.ANEC },
					{ V24.APIC.Id, V24.APIC },
					{ V24.COMM.Id, V24.COMM },
					{ V24.COMR.Id, V24.COMR },
					{ V24.ENCR.Id, V24.ENCR },
					{ V24.EQU2.Id, V24.EQU2 },
					{ V24.ETCO.Id, V24.ETCO },
					{ V24.GEOB.Id, V24.GEOB },
					{ V24.GRID.Id, V24.GRID },
					{ V24.LINK.Id, V24.LINK },
					{ V24.MCDI.Id, V24.MCDI },
					{ V24.MLLT.Id, V24.MLLT },
					{ V24.OWNE.Id, V24.OWNE },
					{ V24.PRIV.Id, V24.PRIV },
					{ V24.PCNT.Id, V24.PCNT },
					{ V24.POPM.Id, V24.POPM },
					{ V24.POSS.Id, V24.POSS },
					{ V24.RBUF.Id, V24.RBUF },
					{ V24.RVA2.Id, V24.RVA2 },
					{ V24.RVRB.Id, V24.RVRB },
					{ V24.SEEK.Id, V24.SEEK },
					{ V24.SIGN.Id, V24.SIGN },
					{ V24.SYLT.Id, V24.SYLT },
					{ V24.SYTC.Id, V24.SYTC },
					{ V24.TALB.Id, V24.TALB },
					{ V24.TBPM.Id, V24.TBPM },
					{ V24.TCOM.Id, V24.TCOM },
					{ V24.TCON.Id, V24.TCON },
					{ V24.TCOP.Id, V24.TCOP },
					{ V24.TDEN.Id, V24.TDEN },
					{ V24.TDLY.Id, V24.TDLY },
					{ V24.TDOR.Id, V24.TDOR },
					{ V24.TDRC.Id, V24.TDRC },
					{ V24.TDRL.Id, V24.TDRL },
					{ V24.TDTG.Id, V24.TDTG },
					{ V24.TENC.Id, V24.TENC },
					{ V24.TEXT.Id, V24.TEXT },
					{ V24.TFLT.Id, V24.TFLT },
					{ V24.TIPL.Id, V24.TIPL },
					{ V24.TIME.Id, V24.TIME },
					{ V24.TIT1.Id, V24.TIT1 },
					{ V24.TIT2.Id, V24.TIT2 },
					{ V24.TIT3.Id, V24.TIT3 },
					{ V24.TKEY.Id, V24.TKEY },
					{ V24.TLAN.Id, V24.TLAN },
					{ V24.TLEN.Id, V24.TLEN },
					{ V24.TMCL.Id, V24.TMCL },
					{ V24.TMED.Id, V24.TMED },
					{ V24.TMOO.Id, V24.TMOO },
					{ V24.TOAL.Id, V24.TOAL },
					{ V24.TOFN.Id, V24.TOFN },
					{ V24.TOLY.Id, V24.TOLY },
					{ V24.TOPE.Id, V24.TOPE },
					{ V24.TOWN.Id, V24.TOWN },
					{ V24.TPE1.Id, V24.TPE1 },
					{ V24.TPE2.Id, V24.TPE2 },
					{ V24.TPE3.Id, V24.TPE3 },
					{ V24.TPE4.Id, V24.TPE4 },
					{ V24.TPOS.Id, V24.TPOS },
					{ V24.TPRO.Id, V24.TPRO },
					{ V24.TPUB.Id, V24.TPUB },
					{ V24.TRCK.Id, V24.TRCK },
					{ V24.TRSN.Id, V24.TRSN },
					{ V24.TRSO.Id, V24.TRSO },
					{ V24.TSOA.Id, V24.TSOA },
					{ V24.TSOP.Id, V24.TSOP },
					{ V24.TSOT.Id, V24.TSOT },
					{ V24.TSRC.Id, V24.TSRC },
					{ V24.TSSE.Id, V24.TSSE },
					{ V24.TSST.Id, V24.TSST },
					{ V24.TXXX.Id, V24.TXXX },
					{ V24.UFID.Id, V24.UFID },
					{ V24.USER.Id, V24.USER },
					{ V24.USLT.Id, V24.USLT },
					{ V24.WCOM.Id, V24.WCOM },
					{ V24.WCOP.Id, V24.WCOP },
					{ V24.WOAF.Id, V24.WOAF },
					{ V24.WOAR.Id, V24.WOAR },
					{ V24.WOAS.Id, V24.WOAS },
					{ V24.WORS.Id, V24.WORS },
					{ V24.WPAY.Id, V24.WPAY },
					{ V24.WPUB.Id, V24.WPUB },
					{ V24.WXXX.Id, V24.WXXX }
				};
			}
		
			/// <summary>
			/// 指定された名前を持つタグ情報を取得します。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
			internal static Id3TagInfo GetInfo( string id )
			{
				Id3TagInfo info;
				return ( V24.Infos.TryGetValue( id, out info ) ? info : null );
			}

			/// <summary>
			/// タグ情報のコレクションを取得します。
			/// </summary>
			/// <returns>タグ情報のコレクション。</returns>
			public static IEnumerable< Id3TagInfo > GetTags()
			{
				return V24.Infos.Values;
			}

			/// <summary>
			/// 指定された名前を持つタグが定義されていることを調べます。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>定義されている場合は true。それ以外は false。</returns>
			internal static bool HasInfo( string id )
			{
				return V24.Infos.ContainsKey( id );
			}
			
			/// <summary>
			/// オーディオの暗号化形式を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ANEC { get; private set; }

			/// <summary>
			/// オーディオ シーケンス ポイント インデックスを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ASPI { get; private set; }

			/// <summary>
			/// ジャケット画像を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo APIC { get; private set; }

			/// <summary>
			/// コメントを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo COMM { get; private set; }

			/// <summary>
			/// コマーシャル フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo COMR { get; private set; }

			/// <summary>
			/// 暗号化手法を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ENCR { get; private set; }

			/// <summary>
			/// イコライザー情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo EQU2 { get; private set; }

			/// <summary>
			/// イベント タイム コードを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo ETCO { get; private set; }

			/// <summary>
			/// パッケージ化された一般的なオブジェクトを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo GEOB { get; private set; }

			/// <summary>
			/// グループ識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo GRID { get; private set; }

			/// <summary>
			/// リンク情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo LINK { get; private set; }

			/// <summary>
			/// 音楽 CD の識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo MCDI { get; private set; }

			/// <summary>
			/// MPEG ロケーション ルックアップ テーブルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo MLLT { get; private set; }

			/// <summary>
			/// 所有権フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo OWNE { get; private set; }

			/// <summary>
			/// プライベートフレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo PRIV { get; private set; }

			/// <summary>
			/// 再生回数を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo PCNT { get; private set; }

			/// <summary>
			/// 人気 ( 1 ～ 255 の数値で表す評価 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo POPM { get; private set; }

			/// <summary>
			/// 同期位置フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo POSS { get; private set; }

			/// <summary>
			/// 推奨バッファサイズを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo RBUF { get; private set; }

			/// <summary>
			/// 相対ボリュームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo RVA2 { get; private set; }

			/// <summary>
			/// リバーブ量を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo RVRB { get; private set; }

			/// <summary>
			/// シークフレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo SEEK { get; private set; }
			
			/// <summary>
			/// 署名フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo SIGN { get; private set; }

			/// <summary>
			/// タイムラインを持つ歌詞情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo SYLT { get; private set; }

			/// <summary>
			/// タイムラインを持つテンポ ( Bit Per Minute ) 情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo SYTC { get; private set; }

			/// <summary>
			/// 曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TALB { get; private set; }

			/// <summary>
			/// テンポ ( Bit Per Minute ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TBPM { get; private set; }

			/// <summary>
			/// 作曲者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCOM { get; private set; }

			/// <summary>
			/// ジャンルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCON { get; private set; }

			/// <summary>
			/// 著作権情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TCOP { get; private set; }

			/// <summary>
			/// エンコード日時を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDEN { get; private set; }

			/// <summary>
			/// プレイリストの遅延時間 ( 曲間の無音時間をスキップするための情報 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDLY { get; private set; }

			/// <summary>
			/// 原曲のリリース日時を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDOR { get; private set; }
			/// <summary>

			/// 録音日時を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDRC { get; private set; }
			
			/// <summary>
			/// リリース日時を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDRL { get; private set; }
			
			/// <summary>
			/// タグ付け日時を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TDTG { get; private set; }

			/// <summary>
			/// ファイルのエンコードをおこなった個人または組織を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TENC { get; private set; }

			/// <summary>
			/// 作詞家を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TEXT { get; private set; }

			/// <summary>
			/// オーディオファイルの形式を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TFLT { get; private set; }

			/// <summary>
			/// 関わった人々の一覧を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIPL { get; private set; }

			/// <summary>
			/// レコーディングに要した時間 ( 形式は HHMM、計 4 桁の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIME { get; private set; }

			/// <summary>
			/// グループの説明を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIT1 { get; private set; }

			/// <summary>
			/// タイトルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIT2 { get; private set; }

			/// <summary>
			/// 副題やタイトルの補足説明を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TIT3 { get; private set; }

			/// <summary>
			/// 曲頭のキー ( C、Am、F♯ といった調の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TKEY { get; private set; }

			/// <summary>
			/// 言語 ( ISO-639-2 として定義された 3 文字の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TLAN { get; private set; }

			/// <summary>
			/// 演奏時間 ( ミリ秒単位 ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TLEN { get; private set; }

			/// <summary>
			/// ミュージシャン クレジット リストを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TMCL { get; private set; }

			/// <summary>
			/// メディア種別 ( 音楽 CD、レコード、...etc ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TMED { get; private set; }

			/// <summary>
			/// 雰囲気を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TMOO { get; private set; }

			/// <summary>
			/// 原曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOAL { get; private set; }

			/// <summary>
			/// 元のファイル名を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOFN { get; private set; }

			/// <summary>
			/// 原曲の作詞者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOLY { get; private set; }

			/// <summary>
			/// 原曲のアーティストを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOPE { get; private set; }

			/// <summary>
			/// 原曲のリリース年を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TORY { get; private set; }

			/// <summary>
			/// ファイルの所有者・ライセンシーを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TOWN { get; private set; }

			/// <summary>
			/// 主なアーティスト、演奏者などを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE1 { get; private set; }

			/// <summary>
			/// レコーディングをおこなった演奏者の追加情報を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE2 { get; private set; }

			/// <summary>
			/// 指揮者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE3 { get; private set; }

			/// <summary>
			/// リミックスや編曲の担当者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPE4 { get; private set; }

			/// <summary>
			/// 曲の属するグループ内の番号 ( スラッシュ区切りの文字列 ) を示すタグ定義を取得します。
			/// 複数枚や複数の組曲で構成されるアルバムの場合、それらの総数と何番目に位置するかを表します。
			/// 例えば 3 枚組のアルバムにおいて、2 枚目に属する曲ならば "2/3" となります。
			/// </summary>
			public static Id3TagInfo TPOS { get; private set; }

			/// <summary>
			/// 制作予告を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPRO { get; private set; }

			/// <summary>
			/// 曲の発行元 ( レーベル ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TPUB { get; private set; }

			/// <summary>
			/// トラック番号 ( 書式は "番号/総数" ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRCK { get; private set; }

			/// <summary>
			/// インターネットラジオ局を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRSN { get; private set; }

			/// <summary>
			/// インターネットラジオ局の所有者を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TRSO { get; private set; }

			/// <summary>
			/// アルバムのソートオーダーを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSOA { get; private set; }
			
			/// <summary>
			/// 演奏者のソートオーダーを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSOP { get; private set; }
		
			/// <summary>
			/// タイトルのソートオーダーを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSOT { get; private set; }

			/// <summary>
			/// 国際標準レコーディングコード（ International Standard Recording Code ) を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSRC { get; private set; }

			/// <summary>
			/// エンコード設定を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSSE { get; private set; }

			/// <summary>
			/// セットのサブタイトルを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TSST { get; private set; }

			/// <summary>
			/// ユーザー定義文字情報フレームを示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo TXXX { get; private set; }

			/// <summary>
			/// ファイルの一意な識別子を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo UFID { get; private set; }

			/// <summary>
			/// 使用条件を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo USER { get; private set; }

			/// <summary>
			/// 歌詞を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo USLT { get; private set; }

			/// <summary>
			/// 販売サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WCOM { get; private set; }

			/// <summary>
			/// 著作情報サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WCOP { get; private set; }

			/// <summary>
			/// オーディオファイルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WOAF { get; private set; }

			/// <summary>
			/// アーティストの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WOAR { get; private set; }

			/// <summary>
			/// 音源の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WOAS { get; private set; }

			/// <summary>
			/// インターネットラジオ局の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WORS { get; private set; }

			/// <summary>
			/// 支払い方法を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WPAY { get; private set; }

			/// <summary>
			/// レーベルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WPUB { get; private set; }

			/// <summary>
			/// ユーザー定義 URL を示すタグ定義を取得します。
			/// </summary>
			public static Id3TagInfo WXXX { get; private set; }

			/// <summary>
			/// フレーム ID をキーとするタグ情報のディクショナリ。
			/// </summary>
			private static Dictionary< string, Id3TagInfo > Infos;
		}
	}
}
