﻿using System;
using System.Text;

namespace Parade.Metadata
{
	/// <summary>
	/// 文字列に関するユーティリティ クラスです。
	/// </summary>
	static class StringUtility
	{
		/// <summary>
		/// 文字列の終端に存在する NULL 文字を削除します。
		/// </summary>
		/// <param name="str">文字列。</param>
		/// <returns>終端の NULL 文字を削除した文字列。</returns>
		public static string TrimLastNull( string str )
		{
			if( str[ str.Length - 1 ] == '\0' )
			{
				// NULL 文字のみなら空文字として扱う
				if( str.Length == 1 ) { return String.Empty; }

				// 終端の NULL 文字をトリミング
				var sb = new StringBuilder( str );
				do
				{
					sb.Remove( sb.Length - 1, 1 );
				} while( sb[ sb.Length - 1 ] == '\0' );

				str = sb.ToString();
			}

			return str;
		}

		/// <summary>
		/// BOM 付きの UTF-16 文字列を表すバイト配列を、文字列に変換します。
		/// </summary>
		/// <param name="value">バイト配列。</param>
		/// <returns>文字列。</returns>
		public static string GetStringUtf16WithBom( byte[] bytes )
		{
			// BOM 分 + 1 文字に満たないなら、空文字として扱う
			if( bytes == null || bytes.Length < 3 ) { return String.Empty; }

			// 適切な BOM が設定されていたら、後続バイトを文字列化する。
			if( bytes[ 0 ] == 0xFE && bytes[ 1 ] == 0xFF )
			{
				return Encoding.BigEndianUnicode.GetString( bytes, 2, bytes.Length - 2 );
			}
			else if( bytes[ 0 ] == 0xFF && bytes[ 1 ] == 0xFE )
			{
				return Encoding.Unicode.GetString( bytes, 2, bytes.Length - 2 );
			}
			else
			{
				return String.Empty;
			}
		}
	}
}
